#include <windows.h>
#include "threads.h"
#include "resource.h"

extern MYTHREAD ThreadInfo []  ;			// thread info array
extern DWORD SleepLength, IdSleepThread  ;	// mailbox to tell thread to sleep

BOOL APIENTRY ThreadDlgProc (HWND hDlg, UINT message, UINT wParam, LONG lParam)
{
	static int LastThread = 0 ;

	switch (message)
	{

		case WM_INITDIALOG:
		{
			int i, Pri ;
/*
Find which threads exist, and enable only those radio buttons.  Check the
one that was checked last
*/
			
			for (i = 0 ; i < NTHREADS ; i ++)
			{
				if (ThreadInfo[i].hThread == 0)
				{
					EnableWindow (GetDlgItem (hDlg, IDC_COLORBASE + i),
						FALSE) ;	
				}
			}
			CheckRadioButton (hDlg, IDC_COLORBASE, IDC_COLORBASE+NTHREADS,
				IDC_COLORBASE+LastThread) ;

/*
find the priority of the checked thread and check the priority's button
*/
			switch (GetThreadPriority (ThreadInfo[LastThread].hThread))
			{
				case THREAD_PRIORITY_IDLE: Pri = IDC_PRI_IDLE ; break ;
				case THREAD_PRIORITY_LOWEST: Pri = IDC_PRI_LOWEST ; break ;
				case THREAD_PRIORITY_BELOW_NORMAL: Pri = IDC_PRI_BELOW ; break ;
				case THREAD_PRIORITY_NORMAL: Pri = IDC_PRI_NORMAL ; break ;
				case THREAD_PRIORITY_ABOVE_NORMAL: Pri = IDC_PRI_ABOVE ; break ;
				case THREAD_PRIORITY_HIGHEST: Pri = IDC_PRI_HIGHEST ; break ;
				case THREAD_PRIORITY_TIME_CRITICAL: Pri = IDC_PRI_CRIT ; break ;
            }
			CheckRadioButton (hDlg, IDC_PRI_IDLE, IDC_PRI_CRIT, Pri) ;
			
			
			return FALSE ;
		}

		case WM_COMMAND:
		{
								
			switch (LOWORD(wParam))
			{
/*
CLOSE button.  Shut down box but take no further action
*/
            	case IDCANCEL:
				{
					EndDialog (hDlg, IDCANCEL) ;
					return TRUE ;
				}

/*
SUSPEND button clicked.  Find out which thread is checked and suspend it
*/

				case IDC_SUSPEND:
				{
					SuspendThread (ThreadInfo[LastThread].hThread) ;
					EndDialog (hDlg, IDOK) ;
					return TRUE ;
				}

/*
RESUME button clicked.  Find out which thread is checked and resume it
*/

				case IDC_RESUME:
				{
					ResumeThread (ThreadInfo[LastThread].hThread) ;
					EndDialog (hDlg, IDOK) ;
					return TRUE ;
                }

/*
TERMINATE button clicked.  Find out which thread is check and terminate it.
Don't forget to close its handle, it will not be removed from memory
until we do
*/
				case IDC_TERMINATE:
				{
					TerminateThread (ThreadInfo[LastThread].hThread, 0) ;
					CloseHandle (ThreadInfo[LastThread].hThread) ;
					ThreadInfo[LastThread].hThread = 0 ;
					ThreadInfo[LastThread].Id = 0 ;
					
					EndDialog (hDlg, IDOK) ;
					return TRUE ;
				}

/*
SLEEP button clicker. You can't sleep a thread other than your own, so we 
have to set a global variable, then modify the thread code to check this
mailbox and put itself to sleep
*/

				case IDC_SLEEP:
				{
					BOOL TransOK ;

					SleepLength = 1000 * GetDlgItemInt (hDlg, IDC_SLEEP_SEC, &TransOK ,FALSE) ; 
					IdSleepThread = ThreadInfo[LastThread].Id ;
					EndDialog (hDlg, IDOK) ;
					return TRUE ;
				}

/*
SET PRIORITY button clicked.  Find which priority radio button is currently 
checked and set the thread's priority to be this level.
*/
				case IDC_SETPRI:
				{
					int i, Pri ;

					for (i = IDC_PRI_IDLE ; i < IDC_PRI_CRIT + 1 ; i ++)
					{
						if (IsDlgButtonChecked (hDlg, i)) 
						{
							break ;
						}
					}

					switch (i)
					{
						case IDC_PRI_IDLE: Pri = THREAD_PRIORITY_IDLE ; break ;
						case IDC_PRI_LOWEST: Pri = THREAD_PRIORITY_LOWEST ; break ;
						case IDC_PRI_BELOW: Pri = THREAD_PRIORITY_BELOW_NORMAL ; break ;
						case IDC_PRI_NORMAL: Pri = THREAD_PRIORITY_NORMAL ; break ;
						case IDC_PRI_ABOVE: Pri = THREAD_PRIORITY_ABOVE_NORMAL ; break ;
						case IDC_PRI_HIGHEST: Pri = THREAD_PRIORITY_HIGHEST ; break ;
						case IDC_PRI_CRIT: Pri = THREAD_PRIORITY_TIME_CRITICAL ; break ;
					}

					SetThreadPriority (ThreadInfo[LastThread].hThread, Pri) ;
					 
					EndDialog (hDlg, IDOK) ;
					return TRUE ;

				}

			
/*
Thread (color) button clicked.  Check its button, then find the thread's
priority and set the priority radio button to reflect this.
*/

				case IDC_BLACK:
				case IDC_RED:
				case IDC_GREEN:
				case IDC_YELLOW:
				case IDC_BLUE:
				case IDC_MAGENTA:
				case IDC_CYAN:
				case IDC_WHITE:
				{
					int OldPri, Pri ;

					LastThread = LOWORD(wParam) - IDC_COLORBASE ;
					CheckRadioButton (hDlg, IDC_BLACK, IDC_WHITE, LOWORD(wParam)) ;
					

					OldPri = GetThreadPriority (ThreadInfo[LastThread].hThread) ; 
					switch (OldPri)
					{
						case THREAD_PRIORITY_IDLE: Pri = IDC_PRI_IDLE ; break ;
						case THREAD_PRIORITY_LOWEST: Pri = IDC_PRI_LOWEST ; break ;
						case THREAD_PRIORITY_BELOW_NORMAL: Pri = IDC_PRI_BELOW ; break ;
						case THREAD_PRIORITY_NORMAL: Pri = IDC_PRI_NORMAL ; break ;
						case THREAD_PRIORITY_ABOVE_NORMAL: Pri = IDC_PRI_ABOVE ; break ;
						case THREAD_PRIORITY_HIGHEST: Pri = IDC_PRI_HIGHEST ; break ;
						case THREAD_PRIORITY_TIME_CRITICAL: Pri = IDC_PRI_CRIT ; break ;
                    
                    }
					CheckRadioButton (hDlg, IDC_PRI_IDLE, IDC_PRI_CRIT, Pri) ;
					return TRUE ;
				}

/*
Priority radio button clicked.  Check the button indicating the priority
that was clicked, but son't set it on the thread until the user pushes the
button
*/
				case IDC_PRI_IDLE:
				case IDC_PRI_LOWEST:
				case IDC_PRI_BELOW:
				case IDC_PRI_NORMAL:
                case IDC_PRI_ABOVE:
				case IDC_PRI_HIGHEST:
				case IDC_PRI_CRIT:
				{
					CheckRadioButton (hDlg, IDC_PRI_IDLE, IDC_PRI_CRIT, LOWORD(wParam)) ;
					return TRUE ;
				}
        	}
		}
	}

	return FALSE ;


}
