/*************************************************************************\
*
*  FUNCTION:  ThreadProc (LPVOID)
*
*  PURPOSE:   A thread procedure which calculates position on the window
*             and draws a colored rectangle.  The color of the rectangle
*             is determined by the input parameter, its border is
*             always black (or whatever the pen color is); and since
*             the window does not paint between rectangle draws, the
*             rectangle leaves a black path in its wake.
*
*  VARIABLES USED:
*
*    - horizontal, vertical:
*             Local int used to indicate the next directional move the
*             rectangle will make.
*
*    - ulx, uly:
*             Local DWORD used for the Upper Left X corner and Upper
*             Upper Left Y position of the rectangle.
*
*    - rect:  A RECT structure used to determin the current size of the
*             window (in case the user resizes it).
*
*    - hdc:   HDC of the rectangle.
*
*    - Time:  A SYSTEMTIME structure.  It's milli-second field is used
*             to create an apparent random starting point for the
*             rectangles.
*
*    -hBrush: A handle to a Brush object, used to set the color of the
*             rectangle.
*
*    -width, height:
*             Local DWORDs used for the width and height of the rectangles.
*
*  CALLED BY:
*
*    MainWndProc();
*
\*************************************************************************/

#include <windows.h>

#define WIDTH 	30
#define HEIGHT 	30
#define BLACK	0x00000000
#define WHITE	RGB(255 * (8 & 0x01), 255 * ((8 & 0x02)>>1), 255 *  ((8 & 0x04)>>2))

DWORD SleepLength, IdSleepThread = 0 ;		// mailbox to tell thread to sleep
extern HWND hMainWnd ;						// window to draw on

VOID ThreadProc (COLORREF *Color)
{
	int HorzDir, VertDir ;
	int xPos, yPos , k;
	RECT r ;

  	HDC  hDC;
  	SYSTEMTIME st ;
  	HANDLE hBrush ;
	HANDLE hBlackBrush;

/*
Get the current time.  Use MOD to get a random x and y starting position
and direction.
*/
  GetSystemTime (&st);                     
  GetClientRect (hMainWnd, &r) ;



  hBlackBrush = CreateSolidBrush((COLORREF) BLACK);


  xPos = (st.wMilliseconds % r.right);  
  yPos = (st.wMilliseconds % r.bottom);  

  HorzDir = 1 ;
  
  if(st.wMilliseconds % 2 == 0)           
  {                                        
     VertDir = 1 ;
  }
  else
  {
     VertDir = -1 ;
  }

/*
The startup parameter we are passed in our stack is a pointer to a COLORREF.
Use it to create the brush we will use to draw.
*/

  hBrush = CreateSolidBrush(*Color);

	

/*
Paint ourselves forever
*/

  for (;;)
  {
  
  /*
Check to see if we're supposed to go to sleep, and for how long
 */
 
 	if (IdSleepThread == GetCurrentThreadId( ))
 	{
		IdSleepThread = 0 ;
 		Sleep (SleepLength) ;	
 	}


/*
Get the client rectangle, check for contact with edges.  If found, change
directions
*/



    
/*
Hit right edge
*/
    if ((xPos + WIDTH) > r.right)  
    {
      xPos = r.right - WIDTH;
      HorzDir = -1;                      
    }

/*
Hit bottom edge
*/
	if ((yPos + HEIGHT) > r.bottom)
    {
      yPos = r.bottom - HEIGHT;           
      VertDir = -1;
    }

/*
Check for left edge
*/
	if (xPos == 0)
	{
		xPos = 1 ;      
      	HorzDir = 1 ;
    }

/*
Check for top edge
*/
	if (yPos == 0)
    {                                     
      yPos = 1 ;
      VertDir = 1 ;
    }

/*
Get a DC for the main window, select this thread's brush into it, draw
and ellipse, release the DC
*/

	GetClientRect (hMainWnd, &r);
    hDC = GetDC (hMainWnd) ;
    SelectObject(hDC, hBlackBrush);
	Ellipse(hDC, xPos , yPos , xPos + WIDTH , yPos + HEIGHT );
  	  	
 
 /*
 Modify position 
 */
  

	xPos += HorzDir ;                     
    yPos += VertDir ;     
	
	SelectObject (hDC, hBrush) ;             
    Ellipse (hDC, xPos, yPos, xPos + WIDTH, yPos + HEIGHT);
    ReleaseDC(hMainWnd, hDC);
	//Sleep (1);
	for (k=1; k<1000000; k++) {};

  }
}
